/*********************************************************************
 * This file is part of the cpplibs suite.
 *
 * Copyright (C) 2001 Topi Menp
 * All rights reserved.
 *
 * This program is free software. You can redistribute and/or modify
 * it under the terms of the free software licence found in the
 * accompanying file "COPYING". The licence terms must always be
 * redistributed with this source file. The above copyright notice
 * must be reproduced in all modified and unmodified copies of this
 * source file.
 *
 * $Revision: 1.4 $
 *********************************************************************/

#include "Operator.h"

#include <List.h>
#include "Expression.h"

namespace util { namespace ariel {

	typedef List<SmartPtr<Computable> > CompList;
	
	SmartPtr<Computable> Comma::operator()(const Computable& c1, const Computable& c2)
		throw (ComputationException&)
	{
		try
			{
				if (_bFirst)
					{			
						SmartPtr<CompList> result(new CompList(2));
						result->addElement(SmartPtr<Computable>(dynamic_cast<Computable*>(c1.clone())));
						result->addElement(SmartPtr<Computable>(dynamic_cast<Computable*>(c2.clone())));
						return SmartPtr<Computable>(result.release());
					}
				else
					{
						const CompList* lst1 = dynamic_cast<const CompList*>(&c1);
						if (!lst1)
					throw ComputationException("Comma:operator()(const Computable&,const Computable&): Left operand must be a list.");
						SmartPtr<CompList> result(new CompList(lst1->getLength()+1));
						for (int i=0;i<lst1->getLength();i++)
							result->addElement(SmartPtr<Computable>(dynamic_cast<Computable*>(lst1->elementAt(i)->clone())));
						result->addElement(SmartPtr<Computable>(dynamic_cast<Computable*>(c2.clone())));
						return SmartPtr<Computable>(result.release());
					}
			}
		catch (NotCloneableException& nce)
			{
				throw ComputationException(nce.getMessage());
			}
	}

	SmartPtr<Computable> Assignment::operator()(const Computable& a, const Computable& b)
		throw (ComputationException&)
	{
		const Variable* var = dynamic_cast<const Variable*>(&a);
		if (var)
			return var->setValue(b);

		typedef List<SmartPtr<Computable> > CompList;
		const CompList* lst = dynamic_cast<const CompList*>(&a);
		if (lst)
			{
				const CompList* lst2 = dynamic_cast<const CompList*>(&b);
				SmartPtr<CompList> result(new CompList(lst->getLength()));
				if (lst2)
					{
						if (lst2->getLength() != lst->getLength())
							throw ComputationException("Assignment::operator()(const Computable&,const Computable&): List lengths differ.");
						for (int i=0;i<lst->getLength();i++)
							result->addElement(operator()(lst->elementAt(i),lst2->elementAt(i)));
					}
				else
					for (int i=0;i<lst->getLength();i++)
						result->addElement(operator()(lst->elementAt(i),b));
				return SmartPtr<Computable>(result.release());
			}
		throw ComputationException("Assignment::operator()(const Computable&,const Computable&): Left operand must be a variable.");
	}

	SmartPtr<Computable> FetchValue::operator() (const Computable& c) throw (ComputationException&)
	{
		const Variable* var = dynamic_cast<const Variable*>(&c);
		if (!var)
			throw ComputationException("FetchValue::operator()(const Computable&): Operand must be a variable.");
		return var->getValue();
	}
}}
